SystemOrganization addCategory: #'Graph-Tests'!
SystemOrganization addCategory: #'Graph-Extended'!
SystemOrganization addCategory: #'Graph-Core'!
SystemOrganization addCategory: #'Graph-Graphics'!
SystemOrganization addCategory: #'Graph-DolphinPort'!
SystemOrganization addCategory: #'Graph-Stochastic'!
SystemOrganization addCategory: #'Graph-StateMachine'!

TestCase subclass: #CostGraphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!CostGraphTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/11/2009 20:52'!
testClearMinimumCostPaths

	| graph v1 v2 |
	graph := CostGraph new.
	v1 := graph createVertexAndAddIt.
	v2 := graph createVertexAndAddIt.
	v1 connectTo: v2 cost: 10.
	
	v2 computeMinimumCostGraphPathsFrom: v1.
	self deny: v1 minimumCostPaths members isEmpty.
	self deny: v2 minimumCostPaths members isEmpty.
	
	graph clearMinimumCostPaths.
	
	self assert: v1 minimumCostPaths members isEmpty.
	self assert: v2 minimumCostPaths members isEmpty.
! !

TestCase subclass: #DijkstraPlatformModelTest
	instanceVariableNames: 'graph a1 a2 b1 b2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!DijkstraPlatformModelTest methodsFor: 'running' stamp: 'hjo 11/9/2009 00:35'!
setUp

	super setUp.
	graph := CostGraph new.
	a1 := graph createNamedVertexAndAddIt: 'A1'.
	a2 := graph createNamedVertexAndAddIt: 'A2'.
	b1 := graph createNamedVertexAndAddIt: 'B1'.
	b2 := graph createNamedVertexAndAddIt: 'B2'.

	a1 connectTo: b1 cost: 2.
	b2 connectTo: a2 cost: 2.
	a1 connect: a2 cost: 5.
	b1 connect: b2 cost: 5.
		! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:34'!
testFromA1ToA2

	self assert: (a2 minimumCostFrom: a1) equals: 5! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:34'!
testFromA1ToB1

	self assert: (b1 minimumCostFrom: a1) equals: 2! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:35'!
testFromA1ToB2

	self assert: (b2 minimumCostFrom: a1) equals: 7! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:37'!
testFromA2ToA1

	self assert: (a1 minimumCostFrom: a2) equals: 5! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:36'!
testFromA2ToB1

	self assert: (b1 minimumCostFrom: a2) equals: 7! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:38'!
testFromA2ToB2

	self assert: (b2 minimumCostFrom: a2) equals: 12! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:38'!
testFromB1ToA1

	self assert: (a1 minimumCostFrom: b1) equals: 12! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:38'!
testFromB1ToA2

	self assert: (a2 minimumCostFrom: b1) equals: 7! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:39'!
testFromB1ToB2

	self assert: (b2 minimumCostFrom: b1) equals: 5! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:39'!
testFromB2ToA1

	self assert: (a1 minimumCostFrom: b2) equals: 7! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:40'!
testFromB2ToA2

	self assert: (a2 minimumCostFrom: b2) equals: 2! !

!DijkstraPlatformModelTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:40'!
testFromB2ToB1

	self assert: (b1 minimumCostFrom: b2) equals: 5! !

TestCase subclass: #DijkstraSubPathTest
	instanceVariableNames: 'graph a b c d e f'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!DijkstraSubPathTest methodsFor: 'running' stamp: 'hjo 11/7/2009 12:57'!
setUp

	super setUp.
	graph := CostGraph new.
	a := graph createNamedVertexAndAddIt: 'A'.
	b := graph createNamedVertexAndAddIt: 'B'.
	c := graph createNamedVertexAndAddIt: 'C'.
	d := graph createNamedVertexAndAddIt: 'D'.
	e := graph createNamedVertexAndAddIt: 'E'.
	f := graph createNamedVertexAndAddIt: 'F'.		

	a connect: b cost: 1.
	b connect: c cost: 1.
	c connect: d cost: 1.
	d connect: e cost: 1.
	e connect: f cost: 1.
	
	e computeMinimumCostGraphPathsFrom: b.	
	self assert: e minimumCost equals: 3
	! !

!DijkstraSubPathTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 18:50'!
testIncludedSubPath

	self assert: (d minimumCostFrom: c) equals: 1! !

!DijkstraSubPathTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 18:51'!
testIncludedSubPathIdentical

	self assert: (e minimumCostFrom: b) equals: 3
! !

!DijkstraSubPathTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 18:51'!
testIncludedSubPathWithSameEndingPoint

	self assert: (e minimumCostFrom: d) equals: 1.
	self assert: (e minimumCostFrom: c) equals: 2.
! !

!DijkstraSubPathTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 18:49'!
testIncludedSubPathWithSameStartingPoint

	self assert: (d minimumCostFrom: b) equals: 2.
	self assert: (c minimumCostFrom: b) equals: 1! !

!DijkstraSubPathTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 18:51'!
testOverlappingSubpathAroundEndingPoint

	self assert: (f minimumCostFrom: d) equals: 2.
	self assert: (f minimumCostFrom: c) equals: 3! !

!DijkstraSubPathTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 18:52'!
testOverlappingSubpathAroundStartingPoint

	self assert: (c minimumCostFrom: a) equals: 2.	
	self assert: (d minimumCostFrom: a) equals: 3! !

!DijkstraSubPathTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 18:52'!
testSubpathIncludingWholePath

	self assert: (f minimumCostFrom: a) equals: 5! !

TestCase subclass: #DijkstraTest
	instanceVariableNames: 'pahaha'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 16:52'!
setUp

	super setUp.
! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 20:07'!
test1

	| graph a b |

	graph := CostGraph new.
	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.
	a connect: b cost: 10.

	b computeMinimumCostGraphPathsFrom: a.
	
	self assert: b minimumCost equals: 10.! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 20:08'!
test2

	| graph a b c |

	graph := CostGraph new.
	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.
	c := graph createVertexAndAddIt.
	a connect: b cost: 10.
	b connect: c cost: 7.

	c computeMinimumCostGraphPathsFrom: a.
	
	self assert: c minimumCost equals: 17.! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 20:08'!
test3

	| graph a b c d |

	graph := CostGraph new.
	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.
	c := graph createVertexAndAddIt.
	d := graph createVertexAndAddIt.
	a connect: b cost: 4.
	b connect: c cost: 7.
	a connect: d cost: 10.
	d connect: c cost: 5.

	c computeMinimumCostGraphPathsFrom: a.

	self assert: c minimumCost equals: 11! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 20:08'!
test4

	| graph a b c d |

	graph := CostGraph new.
	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.
	c := graph createVertexAndAddIt.
	d := graph createVertexAndAddIt.
	a connect: b cost: 4.
	b connect: c cost: 7.
	a connect: d cost: 10.
	d connect: c cost: 5.
	b connect: d cost: 1.
		
	c computeMinimumCostGraphPathsFrom: a.

	self assert: c minimumCost equals: 10! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 20:08'!
test5

	| graph a b d |

	graph := CostGraph new.
	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.
	d := graph createVertexAndAddIt.
	a connect: b cost: 4.
	a connect: d cost: 10.
	b connect: d cost: 1.
	
	d computeMinimumCostGraphPathsFrom: a.

	self assert: d minimumCost equals: 5! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 20:08'!
test6

"http://www.youtube.com/watch?v=8Ls1RqHCOPw"

	| graph a b c d e f g h |
	graph := CostGraph new.

	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.
	c := graph createVertexAndAddIt.
	d := graph createVertexAndAddIt.
	e := graph createVertexAndAddIt.
	f:= graph createVertexAndAddIt.
	g := graph createVertexAndAddIt.
	h := graph createVertexAndAddIt.

	b connectTo: f cost: 10.
	f connectTo: c cost: 10.
	c connectTo: f cost: 50.
	c connectTo: h cost: 20.
	c connectTo: d cost: 10.
	d connectTo: c cost: 10.
	f connectTo: d cost: 40.
	a connectTo: b cost: 20.
	a connectTo: d cost: 80.
	a connectTo: g cost: 90.
	g connectTo: a cost: 20.
	d connectTo: g cost: 20.
	e connectTo: g cost: 30.
	e connectTo: b cost: 50.

	h computeMinimumCostGraphPathsFrom: a.

	self assert: e minimumCostPaths members isEmpty.
	self assert: h minimumCost equals: 60! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 00:35'!
test7

	| graph a b s g |
	graph := CostGraph new.

	s := graph createVertexAndAddIt.
	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.
	g := graph createVertexAndAddIt.

	s connect: a cost: 3.
	s connect: b cost: 4.
	a connect: g cost: 1.
	a connect: g cost: 2.
	b connect: g cost: 1.
	b connect: g cost: 2.

	g computeMinimumCostGraphPathsFrom: s.

	self assert: g minimumCost equals: 4! !

!DijkstraTest methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 00:51'!
testDisconnected

	| graph a b |

	graph := CostGraph new.
	a := graph createVertexAndAddIt.
	b := graph createVertexAndAddIt.

	self should: [b computeMinimumCostGraphPathsFrom: a] raise: Error! !

TestCase subclass: #GraphPathWith1VertexTest
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!GraphPathWith1VertexTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 15:21'!
setUp

	| vertex |
	super setUp.
	vertex := Vertex new.
	path := GraphPathWithOneVertex new.
	path vertex: vertex! !

!GraphPathWith1VertexTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 13:53'!
testExtendingWithEdge

	| linkingVertex linkingEdge |	
	linkingVertex := Vertex new.
	linkingEdge := Edge new.
	linkingEdge connect: path lastVertex and: linkingVertex.
	self shouldnt: [ path extendedTo: linkingVertex via: linkingEdge ] raise: AssertionFailure.
! !

!GraphPathWith1VertexTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 13:53'!
testExtendingWithNoEdge

	self should: [ path extendedTo: Vertex new via: Edge new ] raise: AssertionFailure! !

!GraphPathWith1VertexTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 13:48'!
testFirstAndLastVertex

	self assert: path firstVertex = path lastVertex! !

GraphPathWith1VertexTest subclass: #GraphPathWith2VertexTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!GraphPathWith2VertexTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 12:39'!
setUp

	| vertex edge |
	super setUp.	
	vertex := Vertex new.
	edge := Edge new.
	edge connect: vertex and: path lastVertex.
	
	path := path extendedTo: vertex via: edge.! !

!GraphPathWith2VertexTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 13:48'!
testFirstAndLastVertex

	self deny: path firstVertex = path lastVertex! !

GraphPathWith2VertexTest subclass: #GraphPathWith3VertexTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!GraphPathWith3VertexTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 12:40'!
setUp

	| vertex edge |
	super setUp.	
	vertex := Vertex new.
	edge := Edge new.
	edge connect: vertex and: path lastVertex.
	
	path := path extendedTo: vertex via: edge.! !

TestCase subclass: #GraphTestCase
	instanceVariableNames: 'graph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

GraphTestCase subclass: #DirectionedGraphTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

DirectionedGraphTestCase subclass: #DirectionedGraphBreathFirstDoTest
	instanceVariableNames: 'vertexVisitation edgeVisitation visitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

DirectionedGraphBreathFirstDoTest subclass: #DirectionedGraphBackwardBreathFirstDoTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!DirectionedGraphBackwardBreathFirstDoTest classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:29'!
isAbstract

	^false! !

!DirectionedGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:28'!
setUp

	super setUp.
	visitor beBackwardGoing! !

!DirectionedGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:40'!
test1

	visitor currentVertices: (Array with: (graph vertexNamed: 'A')).

	visitor step.
	self assertVertexVisitionEquals: #('A').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'D' 'G' 'E').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'D' 'G' 'E' 'B' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'D' 'G' 'E' 'B' 'F' 'C' 'H')! !

!DirectionedGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:41'!
test2

	visitor currentVertices: (Array with: (graph vertexNamed: 'B')).

	visitor step.
	self assertVertexVisitionEquals: #('B').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'A').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'A' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'A' 'D' 'E' 'G').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'A' 'D' 'E' 'G' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'A' 'D' 'E' 'G' 'F' 'C' 'H')! !

!DirectionedGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:42'!
test3

	visitor currentVertices: (Array with: (graph vertexNamed: 'F')).

	visitor step.
	self assertVertexVisitionEquals: #('F').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'C' 'H').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'C' 'H' 'G' 'B').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'C' 'H' 'G' 'B' 'A' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'C' 'H' 'G' 'B' 'A' 'D' 'E').

! !

!DirectionedGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:42'!
test4

	visitor currentVertices: (#('A' 'B' 'F') collect: [:each| graph vertexNamed: each]).

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'F' 'D' 'C' 'H').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'F' 'D' 'C' 'H' 'E' 'G').! !

!DirectionedGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:42'!
testRun

	visitor currentVertices: (Array with: (graph vertexNamed: 'A')).
	visitor run.

	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'D' 'F' 'H' 'G')! !

!DirectionedGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:42'!
testRunToVisit

	visitor currentVertices: (Array with: (graph vertexNamed: 'A')).
	visitor runToVisit: (graph vertexNamed: 'E').
	
	self assertVertexVisitionEquals: #('A' 'D' 'G' 'E')! !

!DirectionedGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:38'!
assertVertexVisitionEquals: aCollection

	self assert: ( aCollection asSet equalElementsTo: vertexVisitation )! !

!DirectionedGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/20/2009 15:39'!
setUp

	super setUp.

	vertexVisitation := OrderedCollection new.
	edgeVisitation := OrderedCollection new.

	visitor := BreadthFirstGraphVisitor new.
	visitor vertexOperation: [:vertex| vertexVisitation add: (graph nameOf: vertex) ].
	visitor edgeOperation: [:edge| edgeVisitation add: edge ].! !

DirectionedGraphBreathFirstDoTest subclass: #DirectionedGraphForwardBreathFirstDoTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!DirectionedGraphForwardBreathFirstDoTest classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:30'!
isAbstract

	^false! !

!DirectionedGraphForwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:43'!
test1

	visitor currentVertices: (Array with: (graph vertexNamed: 'A')).

	visitor step.
	self assertVertexVisitionEquals: #('A').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'D' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'D' 'F' 'H' 'G')! !

!DirectionedGraphForwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:43'!
test2

	visitor currentVertices: (Array with: (graph vertexNamed: 'B')).

	visitor step.
	self assertVertexVisitionEquals: #('B').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'C' 'E').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'C' 'E' 'D' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('B' 'C' 'E' 'D' 'F' 'G' 'H' 'A').! !

!DirectionedGraphForwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:44'!
test3

	visitor currentVertices: (Array with: (graph vertexNamed: 'F')).

	visitor step.
	self assertVertexVisitionEquals: #('F').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'E' 'H').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'E' 'H' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'E' 'H' 'D' 'A' 'G').

	visitor step.
	self assertVertexVisitionEquals:  #('F' 'E' 'H' 'D' 'A' 'G' 'B').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'E' 'H' 'D' 'A' 'G' 'B' 'C').! !

!DirectionedGraphForwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:45'!
test4

	visitor currentVertices: (#('A' 'B' 'F') collect: [:each| graph vertexNamed: each]).

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'F' 'C' 'E' 'H').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'F' 'C' 'E' 'H' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'F' 'C' 'E' 'H' 'D' 'G').! !

!DirectionedGraphForwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:45'!
testRun

	visitor currentVertices: (Array with: (graph vertexNamed: 'A')).
	visitor run.

	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'D' 'F' 'H' 'G')! !

!DirectionedGraphForwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:45'!
testRunToVisit

	visitor currentVertices: (Array with: (graph vertexNamed: 'F')).
	visitor runToVisit: (graph vertexNamed: 'A').
	
	self assertVertexVisitionEquals: #('F' 'E' 'H' 'D' 'A' 'G')! !

!DirectionedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/26/2009 20:22'!
setUp

	super setUp.

	'ABCDEFGH' do: [:character| graph addVertex: graph class defaultVertexClass new named: character asString ].
	#('AB' 'BC' 'BE' 'CF' 'FE' 'ED' 'DA' 'GH') 
		do: 
		[:each| graph connectVertexNamed: each first asString toVertexNamed: each second asString ].

	#('DG' 'FH')
		do:
		[:each| graph connectVertexNamed: each first asString andVertexNamed: each second asString ].! !

!DirectionedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 6/25/2011 17:26'!
testConnectedComponents

	self assert: 1 equals: graph connectedComponents size! !

!DirectionedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:23'!
testConnection

	self deny:  ((graph vertexNamed: 'A') isConnectedTo: (graph vertexNamed: 'H')).
	self assert:  ((graph vertexNamed: 'E') isConnectedTo: (graph vertexNamed: 'F'))! !

!DirectionedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:23'!
testIsConnected

	self assert: graph isConnected! !

!DirectionedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:23'!
testIsTree

	self deny: graph isTree! !

GraphTestCase subclass: #DirectionlessGraphTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

DirectionlessGraphTestCase subclass: #DirectionlessGraphBreathFirstDoTest
	instanceVariableNames: 'visitor vertexVisitation edgeVisitation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

DirectionlessGraphBreathFirstDoTest subclass: #DirectionlessGraphBackwardBreathFirstDoTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!DirectionlessGraphBackwardBreathFirstDoTest classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/21/2009 14:32'!
isAbstract

	^false! !

!DirectionlessGraphBackwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 12/5/2009 16:07'!
booboo

	self revisit: 'I think this class is not doing anything.'! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:38'!
assertVertexVisitionEquals: aCollection

	self assert: ( aCollection asSet equalElementsTo: vertexVisitation )! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/20/2009 15:39'!
setUp

	super setUp.

	vertexVisitation := OrderedCollection new.
	edgeVisitation := OrderedCollection new.

	visitor := BreadthFirstGraphVisitor new.
	visitor vertexOperation: [:vertex| vertexVisitation add: (graph nameOf: vertex) ].
	visitor edgeOperation: [:edge| edgeVisitation add: edge ].! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:46'!
test1

	visitor currentVertices: (Array with: (graph vertexNamed: 'A')).

	visitor step.
	self assertVertexVisitionEquals: #('A').

	visitor step.
	self assertVertexVisitionEquals:  #('A' 'B').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'F' 'D' 'G' 'J').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'F' 'D' 'G' 'J' 'K' 'L' 'M' 'H').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'F' 'D' 'G' 'J' 'K' 'L' 'M' 'H' 'N' 'O').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'F' 'D' 'G' 'J' 'K' 'L' 'M' 'H' 'N' 'O' 'P').! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:46'!
test2

	visitor currentVertices: (Array with: (graph vertexNamed: 'C')).

	visitor step.
	self assertVertexVisitionEquals: #('C').

	visitor step.
	self assertVertexVisitionEquals:  #('C' 'B' 'G' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('C' 'B' 'G' 'D' 'A' 'E' 'F' 'M' 'H').
	
	visitor step.
	self assertVertexVisitionEquals: #('C' 'B' 'G' 'D' 'A' 'E' 'F' 'M' 'H' 'J' 'N' 'O').

	visitor step.
	self assertVertexVisitionEquals: #('C' 'B' 'G' 'D' 'A' 'E' 'F' 'M' 'H' 'J' 'N' 'O' 'K' 'L' 'P').
	! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:58'!
test3

	visitor currentVertices: (Array with: (graph vertexNamed: 'F')).

	visitor step.
	self assertVertexVisitionEquals: #('F').

	visitor step.
	self assertVertexVisitionEquals:  #('F' 'B' 'J').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'B' 'J' 'A' 'E' 'C' 'K' 'L').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'B' 'J' 'A' 'E' 'C' 'K' 'L' 'G' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'B' 'J' 'A' 'E' 'C' 'K' 'L' 'G' 'D' 'H' 'M').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'B' 'J' 'A' 'E' 'C' 'K' 'L' 'G' 'D' 'H' 'M' 'N' 'O').

	visitor step.
	self assertVertexVisitionEquals: #('F' 'B' 'J' 'A' 'E' 'C' 'K' 'L' 'G' 'D' 'H' 'M' 'N' 'O' 'P').! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:59'!
test4

	visitor currentVertices: (Array with: (graph vertexNamed: 'H')).

	visitor step.
	self assertVertexVisitionEquals: #('H').

	visitor step.
	self assertVertexVisitionEquals: #('H' 'D' 'M').

	visitor step.
	self assertVertexVisitionEquals: #('H' 'D' 'M' 'C' 'G' 'N' 'O').

	visitor step.
	self assertVertexVisitionEquals: #('H' 'D' 'M' 'C' 'G' 'N' 'O' 'P' 'B').

	visitor step.
	self assertVertexVisitionEquals: #('H' 'D' 'M' 'C' 'G' 'N' 'O' 'P' 'B' 'A' 'E' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('H' 'D' 'M' 'C' 'G' 'N' 'O' 'P' 'B' 'A' 'E' 'F' 'J').

	visitor step.
	self assertVertexVisitionEquals: #('H' 'D' 'M' 'C' 'G' 'N' 'O' 'P' 'B' 'A' 'E' 'F' 'J' 'K' 'L').! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 15:00'!
test5

	visitor currentVertices: (Array with: (graph vertexNamed: 'N')).

	visitor step.
	self assertVertexVisitionEquals: #('N').

	visitor step.
	self assertVertexVisitionEquals: #('N' 'M').

	visitor step.
	self assertVertexVisitionEquals: #('N' 'M' 'G' 'H' 'O').

	visitor step.
	self assertVertexVisitionEquals: #('N' 'M' 'G' 'H' 'O' 'P' 'C' 'D').

	visitor step.
	self assertVertexVisitionEquals: #('N' 'M' 'G' 'H' 'O' 'P' 'C' 'D' 'B').

	visitor step.
	self assertVertexVisitionEquals: #('N' 'M' 'G' 'H' 'O' 'P' 'C' 'D' 'B' 'A' 'E' 'F').

	visitor step.
	self assertVertexVisitionEquals: #('N' 'M' 'G' 'H' 'O' 'P' 'C' 'D' 'B' 'A' 'E' 'F' 'J').

	visitor step.
	self assertVertexVisitionEquals: #('N' 'M' 'G' 'H' 'O' 'P' 'C' 'D' 'B' 'A' 'E' 'F' 'J' 'K' 'L').! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 15:00'!
test6

	visitor currentVertices: (#('A' 'H') collect: [:name| graph vertexNamed: name]).

	visitor step.
	self assertVertexVisitionEquals: #('A' 'H').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'H' 'B' 'D' 'M').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'H' 'B' 'D' 'M' 'E' 'F' 'C' 'G' 'N' 'O').

	visitor step.
	self assertVertexVisitionEquals: #('A' 'H' 'B' 'D' 'M' 'E' 'F' 'C' 'G' 'N' 'O' 'J' 'P').! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 15:05'!
testRun

	visitor currentVertices: (Array with: (graph vertexNamed: 'A')).
	visitor run.

	self assertVertexVisitionEquals: #('A' 'B' 'C' 'E' 'F' 'D' 'G' 'J' 'K' 'L' 'M' 'H' 'N' 'O' 'P')! !

!DirectionlessGraphBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 15:05'!
testRunToVisit

	visitor currentVertices: (Array with: (graph vertexNamed: 'H')).
	visitor runToVisit: (graph vertexNamed: 'B').

	self assertVertexVisitionEquals: #('H' 'D' 'M' 'C' 'G' 'N' 'O' 'P' 'B')! !

DirectionlessGraphBreathFirstDoTest subclass: #DirectionlessGraphForwardBreathFirstDoTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!DirectionlessGraphForwardBreathFirstDoTest classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/21/2009 14:18'!
isAbstract

	^false! !

!DirectionlessGraphForwardBreathFirstDoTest methodsFor: 'as yet unclassified' stamp: 'hjo 12/5/2009 16:07'!
booboo

	self revisit: 'I think this class is not doing anything.'! !

!DirectionlessGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/26/2009 20:23'!
setUp

	super setUp.
	
	'ABCDEFGHJKLMNOP' do: [:each| graph addVertex: graph class defaultVertexClass new named: (String with: each) ].
	#('AB' 'BC' 'CD' 'BE' 'BF' 'CG' 'DH' 'EJ' 'FJ' 'GM' 'HM' 'JK' 'JL' 'MN' 'MO' 'OP') 
		do: [:each| graph connectVertexNamed: each first asString andVertexNamed: each second asString ].! !

!DirectionlessGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 6/25/2011 17:26'!
testConnectedComponents

	self assert: 1 equals: graph connectedComponents size! !

!DirectionlessGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:24'!
testConnectionTesting

	self assert: ( (graph vertexNamed: 'B') isConnectedTo: (graph vertexNamed: 'A')).
	self deny: ( (graph vertexNamed: 'B') isConnectedTo: (graph vertexNamed: 'P')).
	self assert: ( (graph vertexNamed: 'F') isConnectedTo: (graph vertexNamed: 'J')).
	self deny: ( (graph vertexNamed: 'F') isConnectedTo: (graph vertexNamed: 'E')).
	self assert: ( (graph vertexNamed: 'M') isConnectedTo: (graph vertexNamed: 'N')).
	self deny: ( (graph vertexNamed: 'M') isConnectedTo: (graph vertexNamed: 'D')).
	self assert: ( (graph vertexNamed: 'C') isConnectedTo: (graph vertexNamed: 'G')).! !

!DirectionlessGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 6/25/2011 17:27'!
testEdgeCount

	self assert: 16 equals: graph edges size! !

!DirectionlessGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:24'!
testIsTree

	self deny: graph isTree! !

!GraphTestCase classSide methodsFor: 'testing' stamp: 'hjo 9/18/2009 20:21'!
isAbstract

	^true! !

!GraphTestCase methodsFor: 'running' stamp: 'hjo 9/26/2009 20:21'!
setUp

	super setUp.
	graph := NamedGraph new.! !

!GraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/26/2009 20:15'!
testNodeNames

	| v |
	v := graph getOrAndNewVertexNamed: 'a'.
	self assert: (graph nameOf: v) equals: 'a'.
	self assert: (graph hasVertexNamed: 'a').
	self deny: (graph hasVertexNamed: 'b').
	
	self assert: (graph vertexNamed: 'a') equals: v.
	
	graph name: v as: 'c'.
	self assert: (graph vertexNamed: 'c') equals: v.
	self assert: (graph vertexNames includes: 'c').
	self assert: (graph nodeNames includes: 'c')! !

GraphTestCase subclass: #TreeLikeGraphTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!TreeLikeGraphTestCase classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:25'!
isAbstract

	^false! !

!TreeLikeGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/26/2009 20:24'!
setUp

	super setUp.
	'ABCDEFG' do: [:each| graph addVertex: graph class defaultVertexClass new named: each asString ].

	#('AB' 'BC' 'BD' 'AE' 'EF' 'EG') do: [:each| graph connectVertexNamed: each first asString andVertexNamed: each second asString ]! !

!TreeLikeGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 6/25/2011 17:27'!
testConnectedComponents

	self assert: 1 equals: graph connectedComponents size! !

!TreeLikeGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:25'!
testIsTree

	self assert: graph isTree! !

GraphTestCase subclass: #UnconnectedGraphTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Tests'!

!UnconnectedGraphTestCase classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:26'!
isAbstract

	^false! !

!UnconnectedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/26/2009 20:25'!
setUp

	super setUp.
	
	'ABCDEFG' do: [:each| graph addVertex: graph class defaultVertexClass new named: (String with: each) ].
	#('AB' 'BC' 'CA' 'DE' 'EG' 'GF' 'FD') 
		do: [:each| graph connectVertexNamed: each first asString andVertexNamed: each second asString ].
! !

!UnconnectedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 6/25/2011 17:28'!
testConnectedComponents

	self assert: #(3 4) asSortedCollection equals: (graph connectedComponents collect: [:subGraph| subGraph vertices size ]) asSortedCollection! !

!UnconnectedGraphTestCase methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:26'!
testIsTree

	self deny: graph isTree! !

Object subclass: #BreadthFirstGraphVisitor
	instanceVariableNames: 'currentVertices vertexOperation edgeOperation visitedVertices nextVerticesToVisit nextEdgesToVisit edgeAccessingMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Core'!

!BreadthFirstGraphVisitor methodsFor: 'state changing' stamp: 'hjo 9/21/2009 13:57'!
beBackwardGoing

	self revisit: 'I dont think anyone would change forward backward direcction once the object is created. I am trying to say that simply extracting two concrete subclass out of this class, and  making this class abstract will be good enough and simple as well.'.

	self edgeAccessingMethod: #edgesThatCanEnter! !

!BreadthFirstGraphVisitor methodsFor: 'state changing' stamp: 'hjo 9/21/2009 13:57'!
beForwardGoing

	self revisit: 'I dont think anyone would change forward backward direcction once the object is created. I am trying to say that simply extracting two concrete subclass out of this class, and  making this class abstract will be good enough and simple as well.'.
	self edgeAccessingMethod: #edgesThatCanLeave! !

!BreadthFirstGraphVisitor methodsFor: 'running' stamp: 'hjo 9/20/2009 15:40'!
breadthFirstSearch

	self currentVertices do: 
		[ :vertex| 
			| accessingEdgesToNextVerticesToVist |
			accessingEdgesToNextVerticesToVist := 
				(vertex perform: self edgeAccessingMethod)
					reject: [:edge| self visitedVertices includes: (edge otherEndPointOf: vertex) ].
			accessingEdgesToNextVerticesToVist 
				do: 
				[:each| 
					self nextVerticesToVisit add: (each otherEndPointOf: vertex)
				].
			self nextEdgesToVisit addAll: accessingEdgesToNextVerticesToVist
		]! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:05'!
currentVertices

	^currentVertices! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:05'!
currentVertices: anArray 

	^currentVertices := anArray asSet! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:05'!
edgeAccessingMethod

	edgeAccessingMethod isNil ifTrue: [ edgeAccessingMethod := #edgesThatCanLeave ].
	^edgeAccessingMethod! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:05'!
edgeAccessingMethod: aSymbol

	edgeAccessingMethod := aSymbol! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:05'!
edgeOperation

	edgeOperation isNil ifTrue: [ edgeOperation := DeafObject new ].
	^edgeOperation! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:06'!
edgeOperation: aBlockClosure 

	^edgeOperation := aBlockClosure 
! !

!BreadthFirstGraphVisitor methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:06'!
isBackwardGoing

	self edgeAccessingMethod = #edgesThatCanEnter! !

!BreadthFirstGraphVisitor methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:06'!
isForwardGoing

	self edgeAccessingMethod = #edgesThatCanLeave! !

!BreadthFirstGraphVisitor methodsFor: 'querying' stamp: 'hjo 9/20/2009 15:40'!
nextEdgesToVisit 

	nextEdgesToVisit isNil 
		ifTrue: [ 
			nextEdgesToVisit := Set new.
			self breadthFirstSearch
		].
	^nextEdgesToVisit! !

!BreadthFirstGraphVisitor methodsFor: 'querying' stamp: 'hjo 9/20/2009 15:40'!
nextVerticesToVisit

	nextVerticesToVisit isNil 
		ifTrue: [ 
			nextVerticesToVisit := Set new.
			self breadthFirstSearch
		].
	^nextVerticesToVisit
! !

!BreadthFirstGraphVisitor methodsFor: 'running' stamp: 'hjo 12/7/2009 16:31'!
run

	self revisit: 'This method should be renamed something like "runUntilThereIsNoVertexToVisit" '.
	"Use #runToVisit: if you want to stop the algorithm when you reach the given vertex."
	[ self step. self nextVerticesToVisit isEmpty ] whileFalse! !

!BreadthFirstGraphVisitor methodsFor: 'running' stamp: 'hjo 9/21/2009 14:03'!
runToVisit: aVertex

	[ self step. self visitedVertices includes: aVertex ] whileFalse! !

!BreadthFirstGraphVisitor methodsFor: 'running' stamp: 'hjo 9/18/2009 16:07'!
step

	"Based on BFS(Breath First Search) algorithm"
	self currentVertices do: [:each| self vertexOperation value: each.  self visitedVertices add: each ].
	nextVerticesToVisit := nil.
	nextEdgesToVisit := nil.
	self nextEdgesToVisit do: [:each| self edgeOperation value: each ].
	self currentVertices: self nextVerticesToVisit! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:07'!
vertexOperation

	vertexOperation isNil ifTrue: [ vertexOperation := DeafObject new ].
	^vertexOperation! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:07'!
vertexOperation: aBlockClosure 

	^vertexOperation := aBlockClosure 
! !

!BreadthFirstGraphVisitor methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:07'!
visitedVertices

	visitedVertices isNil ifTrue: [ visitedVertices := Set new ].
	^visitedVertices! !

Object subclass: #DeafObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-DolphinPort'!

!DeafObject commentStamp: 'hjo 9/18/2009 16:13' prior: 0!
From Dolphin. Squeak is missing it.!

!DeafObject methodsFor: 'as yet unclassified' stamp: 'hjo 9/19/2009 15:44'!
doesNotUnderstand: aSymbol

	"neglect all messages"! !

Object subclass: #Dijkstra
	instanceVariableNames: 'priorityQueue source goalVertex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Core'!

!Dijkstra classSide methodsFor: 'as yet unclassified' stamp: 'hjo 11/12/2009 16:37'!
computeMinimumCostGraphPathsFrom: aCostVertex1 to: aCostVertex2 

	| instance |
	instance := self new.
	instance startingVertex: aCostVertex1.
	instance goalVertex: aCostVertex2.
	instance gogogo.
	aCostVertex2 minimumCostPathsClub minimum isNil ifTrue: [ self error: 'Could not reach the goal vertex.' ]! !

!Dijkstra methodsFor: 'accessing' stamp: 'hjo 11/11/2009 00:02'!
goalVertex: aCostVertex

	goalVertex := aCostVertex! !

!Dijkstra methodsFor: 'operation' stamp: 'hjo 11/11/2009 22:39'!
gogogo
	
	| topPriorityVertex myPath neighbor |
	self initializePriorityQueue.
	priorityQueue do: [:each| each initializeMinimumCostPathsClubWithSource: source ].
	
	source minimumCostPathsClub put: (GraphPathWithOneVertex vertex: source).

	"while loop"
	[ priorityQueue isEmpty ] whileFalse: [
		topPriorityVertex := priorityQueue remove: self smallestMinimumCostVertexInPriorityQueue.
		topPriorityVertex minimumCostPathsClub members isEmpty
			ifTrue: [self halt]
			ifFalse: [
				myPath := topPriorityVertex minimumCostPathsClub members anyOne.
				topPriorityVertex edgesThatCanLeave do: [:edge| 
					neighbor := edge otherEndPointOf: topPriorityVertex.
					neighbor minimumCostPathsClub put: 
						(myPath extendedTo: neighbor via: edge)
				].
			].
	]! !

!Dijkstra methodsFor: 'initialize-release' stamp: 'hjo 11/11/2009 22:30'!
initializePriorityQueue
	
	| b |
	b := BreadthFirstGraphVisitor new.
	b currentVertices: { source }.
	b run.
	priorityQueue := b visitedVertices asOrderedCollection! !

!Dijkstra methodsFor: 'finding' stamp: 'hjo 11/11/2009 22:36'!
smallestMinimumCostVertexInPriorityQueue

	^((priorityQueue select: [:each| each minimumCost notNil ]) asSortedCollection: [:vertex| vertex minimumCost ] ascendingSortBlock) first! !

!Dijkstra methodsFor: 'accessing' stamp: 'hjo 11/11/2009 00:00'!
source: aCostVertex

	source := aCostVertex! !

!Dijkstra methodsFor: 'accessing' stamp: 'hjo 11/11/2009 00:01'!
startingVertex: aCostVertex

	self source: aCostVertex! !

Object subclass: #Edge
	instanceVariableNames: 'endPoint1 endPoint2 hasDirection masterObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Core'!

!Edge commentStamp: 'hjo 9/18/2009 16:13' prior: 0!
Edge connect two vertices. Edge sand Vertices form a graph mathematically.!

Edge subclass: #CostEdge
	instanceVariableNames: 'cost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!CostEdge methodsFor: 'as yet unclassified' stamp: 'hjo 9/19/2009 16:38'!
cost

	^cost! !

!CostEdge methodsFor: 'as yet unclassified' stamp: 'hjo 9/19/2009 16:38'!
cost: anObject

	cost := anObject! !

!CostEdge methodsFor: 'as yet unclassified' stamp: 'hjo 9/25/2009 18:25'!
extraPrintString

	^',cost: ', cost printString! !

!Edge methodsFor: 'comparing' stamp: 'hjo 9/19/2009 15:42'!
= anEdge
	
"Two edges connecting the same two points i,j are regarded same. If the edge has direction, the test becomes more tight."
	self hasDirection
		ifTrue: [ 
			^anEdge hasDirection and:
			[(self endPoint1 = anEdge endPoint1) and:
			[self endPoint2 = anEdge endPoint2]]]
		ifFalse: [^self endPoints equalElementsTo: anEdge endPoints]
! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:14'!
atLeastOneTerminalConnected

	^endPoint1 notNil or: [endPoint2 notNil]! !

!Edge methodsFor: 'connecting' stamp: 'hjo 9/18/2009 16:14'!
basicConnect: aVertex and: bVertex

	endPoint1 := aVertex.
	aVertex addEdge: self.

	endPoint2 := bVertex.
	bVertex addEdge: self.

	self onConnect! !

!Edge methodsFor: 'state changing' stamp: 'hjo 9/18/2009 16:14'!
beDirected

	hasDirection := true! !

!Edge methodsFor: 'state changing' stamp: 'hjo 9/18/2009 16:14'!
beUndirected

	hasDirection := false! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:14'!
bothTerminalConnected

	^endPoint1 notNil and: [ endPoint2 notNil ]! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/21/2009 15:54'!
canFlowFrom: aVertex to: aVertex2 

	^self isDirectedEdge
		ifTrue: [ self initialPoint = aVertex and: [ self terminalPoint = aVertex2 ] ]
		ifFalse: [ self connects: aVertex and: aVertex2 ]! !

!Edge methodsFor: 'connecting' stamp: 'hjo 9/18/2009 16:14'!
connect: aVertex and: bVertex

	self beUndirected.
	self basicConnect: aVertex and: bVertex! !

!Edge methodsFor: 'connecting' stamp: 'hjo 9/18/2009 16:15'!
connectFrom: aVertex to: bVertex
"Connects vertices with direction"
	
	self beDirected.	
	self basicConnect: aVertex and: bVertex! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:15'!
connects: aVertex

	^(endPoint1 == aVertex) or: [endPoint2 == aVertex]! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:15'!
connects: aVertex and: bVertex

	|endPoints|
	endPoints := self endPoints.
	^(endPoints includes: aVertex) and: [endPoints includes: bVertex]
! !

!Edge methodsFor: 'connecting' stamp: 'hjo 9/18/2009 16:15'!
disconnect

	self endPoint1 removeEdge: self.
	self endPoint1: nil.
	self endPoint2 removeEdge: self.
	self endPoint2: nil.! !

!Edge methodsFor: 'drawing' stamp: 'hjo 9/18/2009 16:15'!
drawOn: aCanvas

	aCanvas moveTo: endPoint1 position asPoint.
	aCanvas lineTo: endPoint2 position asPoint.! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:15'!
endPoint1

	^endPoint1! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:15'!
endPoint1: aVertex

	^endPoint1 := aVertex! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:16'!
endPoint2

	^endPoint2! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:16'!
endPoint2: aVertex

	^endPoint2 := aVertex! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 15:55'!
endPoints

	| ep |
	ep := Set new.
	endPoint1 ifNotNil: [ ep add: endPoint1 ].
	endPoint2 ifNotNil: [ ep add: endPoint2 ].
	^ep! !

!Edge methodsFor: 'error handling' stamp: 'hjo 9/18/2009 16:16'!
errorUndirectedEdge

	self error: 'Undirected Edge cannot do that'! !

!Edge methodsFor: 'error handling' stamp: 'hjo 9/18/2009 16:16'!
errorVertexMissing

	self error: 'The vertex is not an end point of this edge'! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:16'!
hasDirection

	^self isDirectedEdge! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:17'!
initialPoint

	self hasDirection ifFalse: [ self errorUndirectedEdge ].
	^endPoint1
! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:17'!
initialPoint: aVertex

	self hasDirection ifFalse: [ self errorUndirectedEdge ].
	^endPoint1 := aVertex
! !

!Edge methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 16:17'!
initialize

	super initialize.
	hasDirection := false! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:17'!
isDirectedEdge

	^hasDirection = true
! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:17'!
isLoop

	^endPoint1 == endPoint2! !

!Edge methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:17'!
isMultipleEdgeWith: anEdge

	^self endPoints = anEdge endPoints! !

!Edge methodsFor: 'connecting' stamp: 'hjo 9/18/2009 16:17'!
letGo: aVertex

"Remove reference to aVertex. End points must be determined ahead."
	(self endPoint1 == aVertex)
		ifTrue: [ ^self endPoint1: nil ].
	(self endPoint2 == aVertex)
		ifTrue: [ ^self endPoint2: nil ].
	self errorVertexMissing! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 18:51'!
masterObject

	^masterObject! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 18:51'!
masterObject: anObject

	masterObject := anObject! !

!Edge methodsFor: 'event handling' stamp: 'hjo 9/18/2009 16:18'!
onConnect

	"After connection established"! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:18'!
otherEndPointOf: aVertex

	(self endPoint1 == aVertex)
		ifTrue: [ ^self endPoint2 ].
	(self endPoint2 == aVertex)
		ifTrue: [ ^self endPoint1 ].
	self errorVertexMissing	! !

!Edge methodsFor: 'drawing' stamp: 'hjo 9/18/2009 16:18'!
rectangleToInvalidate

	^(endPoint1 position asPoint corner: endPoint2 position asPoint) normalize! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:18'!
terminalPoint

	self hasDirection ifFalse: [ self errorUndirectedEdge ].
	^endPoint2! !

!Edge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:18'!
terminalPoint: aVertex

	self hasDirection ifFalse: [ self errorUndirectedEdge ].
	^endPoint2 := aVertex! !

Edge subclass: #FlowEdge
	instanceVariableNames: 'capacity flow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!FlowEdge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 16:43'!
capacity

	^capacity! !

!FlowEdge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 16:44'!
capacity: aNumber

	^capacity := aNumber! !

!FlowEdge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 16:46'!
flow

	^flow! !

!FlowEdge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 16:46'!
flow: aFlowNumber
	
	capacity notNil
		ifTrue: [^flow := aFlowNumber min: capacity]
		ifFalse: [^flow := aFlowNumber]! !

!FlowEdge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 16:47'!
flowEntering: aVertex

	(aVertex == self endPoint1)
		ifTrue: [ ^self flow negated ].
	(aVertex == self endPoint2)
		ifTrue: [ ^self flow ].
	self errorVertexMissing
! !

!FlowEdge methodsFor: 'testing' stamp: 'hjo 9/19/2009 16:48'!
hasFlowLimit

	^self flow notNil! !

!FlowEdge methodsFor: 'testing' stamp: 'hjo 9/19/2009 16:48'!
isFlowOffLimit

	^(self hasDirection)
		ifTrue: [ flow > capacity ]
		ifFalse: [ flow abs > capacity abs ]! !

!FlowEdge methodsFor: 'accessing' stamp: 'hjo 9/19/2009 16:49'!
noFlowLimit

	self flow: nil! !

Edge subclass: #GraphDistance
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!GraphDistance commentStamp: '<historical>' prior: 0!
Stores distance between its two end point in the instance variable named "cost".
Distance is symentically undirected edge.!

!GraphDistance methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:30'!
onConnect

	self updateCost! !

!GraphDistance methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:30'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: cost printString;
		nextPut: $)! !

!GraphDistance methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:31'!
updateCost

	cost := (endPoint1 position - endPoint2 position) normalized! !

Edge subclass: #PlotEdge
	instanceVariableNames: 'type color thickness'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Graphics'!

!PlotEdge methodsFor: 'state changing' stamp: 'hjo 9/18/2009 19:31'!
beDashed

	type := #dashed! !

!PlotEdge methodsFor: 'state changing' stamp: 'hjo 9/18/2009 19:32'!
beSolid

	type := #solid! !

!PlotEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:32'!
color
	"Private - Answer the value of the receiver's ''color'' instance variable."

	^color! !

!PlotEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:32'!
color: anObject
	"Private - Set the value of the receiver's ''color'' instance variable to the argument, anObject."

	color := anObject! !

!PlotEdge methodsFor: 'graphics' stamp: 'hjo 9/18/2009 19:32'!
drawOn: aCanvas

"	Dolphin4 Code
	|oPen|
	oPen := aCanvas pen: (Pen withStyle: self penStyle width: thickness color: color).
	super drawOn: aCanvas.
	aCanvas pen: oPen"! !

!PlotEdge methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 19:33'!
initialize

	super initialize.
	type := #solid.
	color := Color red.
	thickness := 2.! !

!PlotEdge methodsFor: 'graphics' stamp: 'hjo 9/18/2009 19:33'!
penStyle

"	Dolphin4 code
	(type = #dashed) ifTrue: [^PS_DASH].
	(type = #solid) ifTrue: [^PS_SOLID].
	^PS_DASH"! !

!PlotEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:33'!
thickness

	^thickness! !

!PlotEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:33'!
thickness: anInteger

	thickness := anInteger! !

!PlotEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:33'!
type

	^type! !

!PlotEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:33'!
type: aSymbol

	type := aSymbol! !

Edge subclass: #StateEdge
	instanceVariableNames: 'output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-StateMachine'!

!StateEdge methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:35'!
output

	^output! !

!StateEdge methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:35'!
output: anObject

	output := anObject! !

StateEdge subclass: #StochasticEdge
	instanceVariableNames: 'probability'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Stochastic'!

!StochasticEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:35'!
probability

	^probability! !

!StochasticEdge methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:35'!
probability: aNumber

	probability := aNumber! !

Object subclass: #Graph
	instanceVariableNames: 'vertices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Core'!

!Graph commentStamp: 'hjo 9/19/2009 18:02' prior: 0!
'We meet graphs in different fields under different names: as "networks" in electrical engineering, "structures" in civil engineergin, "molecular structres" in chemistry, "organizational structures" in economics, "sociograms," "road maps," "telecommunications networks," and so on.

See Also: Vertex, Edge, Path'

Edges are not stored in me. Edges are indirectly stored via vertices.!

Graph subclass: #CostGraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!CostGraph classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/21/2009 14:46'!
costedExample

"A weighted undirected graph"
	| a b c d e f g h i j k l m graph vertexClass |
	vertexClass := CostVertex.
	graph := self new.
	graph
		addVertex: (a := vertexClass new);
		addVertex: (b := vertexClass new);
		addVertex: (c := vertexClass new);
		addVertex: (d := vertexClass new);
		addVertex: (e := vertexClass new);
		addVertex: (f := vertexClass new);
		addVertex: (g := vertexClass new);
		addVertex: (h := vertexClass new);
		addVertex: (i := vertexClass new);
		addVertex: (j := vertexClass new);
		addVertex: (k := vertexClass new);
		addVertex: (l := vertexClass new);
		addVertex: (m := vertexClass new).

	a connect: b cost: 1.
	a connect: f cost: 2.
	a connect: g cost: 6.
	b connect: d cost: 2.
	b connect: c cost: 1.
	b connect: e cost: 4.
	d connect: f cost: 1.
	d connect: e cost: 2.
	c connect: e cost: 4.
	f connect: e cost: 2.
	f connect: l cost: 2.
	g connect: e cost: 1.
	g connect: h cost: 3.
	g connect: j cost: 1.
	g connect: l cost: 5.
	h connect: i cost: 2.
	j connect: k cost: 1.
	j connect: l cost: 3.
	i connect: k cost: 1.
	j connect: m cost: 2.
	l connect: m cost: 1.
	^graph	
! !

!CostGraph classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 14:28'!
defaultVertexClass

	^CostVertex! !

!CostGraph methodsFor: 'as yet unclassified' stamp: 'hjo 9/26/2009 17:39'!
clearMinimumCostPaths

	vertices do: [:each| each clearMinimumCostPaths ]! !

!CostGraph methodsFor: 'as yet unclassified' stamp: 'hjo 11/7/2009 13:02'!
createNamedVertexAndAddIt: aString
	
	| newVertex |
	newVertex := NamedCostVertex new.
	self addVertex: newVertex.
	newVertex vertexName: aString.
	^newVertex! !

!Graph classSide methodsFor: 'instance creation' stamp: 'hjo 9/18/2009 16:57'!
createWithFullyConnectedVertices: aCollection

	|unconnected graph|
	graph := self vertices: aCollection asSet.
	unconnected := aCollection asSet.
	graph vertices do: [:vertex| |toConnect|
		toConnect := unconnected - vertex neighbors.
		toConnect remove: vertex.
		vertex connectAll: toConnect ].
	^graph! !

!Graph classSide methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:57'!
defaultVertexClass

	^Vertex! !

!Graph classSide methodsFor: 'examples' stamp: 'hjo 9/18/2009 16:57'!
example

	| graph |
	graph := self new.
	graph addVerticesNamed: #(a b c d e f g h i j k l m).
	(graph vertexNamed: #a) 
		connectAll: (graph verticesNamed: #(b c g f)).
	(graph vertexNamed: #f)
		connectAll: (graph verticesNamed: #(d e)).
	(graph vertexNamed: #h)
		connectAll: (graph verticesNamed: #(i g)).
	(graph vertexNamed: #j)
		connectAll: (graph verticesNamed: #(k m l g)).
	(graph vertexNamed: #l)
		connectAll: (graph verticesNamed: #(m g)).
	(graph vertexNamed: #e)
		connectAll: (graph verticesNamed: #(d g)).
	(graph vertexNamed: #c)
		connect: (graph vertexNamed: #g).
	^graph! !

!Graph classSide methodsFor: 'instance creation' stamp: 'hjo 9/18/2009 16:58'!
vertices: aVertexCollection

	^self new
		vertices: aVertexCollection;
		yourself! !

!Graph methodsFor: 'vertex' stamp: 'hjo 9/18/2009 16:21'!
addAllVertices: aCollection

	^self vertices addAll: aCollection! !

!Graph methodsFor: 'vertex' stamp: 'hjo 9/18/2009 16:21'!
addVertex: aVertex

	^self vertices add: aVertex! !

!Graph methodsFor: 'data structures' stamp: 'hjo 9/18/2009 16:22'!
adjacencyMatrix
"By definition, no vertex is considered to be adjacents to itself. The adjacencyMatrix is always symmetric. The adjacency matrix of a graph is generally much smaller than thesocalled incidence matrix"
	self notImplemented! !

!Graph methodsFor: 'data structures' stamp: 'hjo 9/18/2009 16:30'!
aminimumSpanningTree

"Kruscal's Algorithm"
	self notYetImplemented! !

!Graph methodsFor: 'visiting operation' stamp: 'hjo 12/1/2009 19:11'!
backBreadthFirstDo: aVertexProcessingBlock startingAt: aVertex
"Based on BFS(Breath First Search) algorithm"

	self
		backBreadthFirstDo: aVertexProcessingBlock
		startingAt: aVertex
		visitedVertices: (Set new: self vertices size) ! !

!Graph methodsFor: 'visiting operation' stamp: 'hjo 9/18/2009 16:22'!
backBreadthFirstDo: aVertexProcessingBlock startingAt: aVertex visitedVertices: visited
"Based on BFS(Breath First Search) algorithm"

	self
		backBreadthFirstDo: aVertexProcessingBlock
		startingVertices: (Set with: aVertex)
		visitedVertices: visited! !

!Graph methodsFor: 'visiting operation' stamp: 'hjo 9/18/2009 16:23'!
backBreadthFirstDo: aVertexProcessingBlock startingVertices: aSet1 visitedVertices: aSet2
"Based on BFS(Breath First Search) algorithm"

	self backBreadthFirstDoVertex: aVertexProcessingBlock doEdge: [:edge|] startingVertices: aSet1 visitedVertices: aSet2! !

!Graph methodsFor: 'visiting operation' stamp: 'hjo 9/18/2009 16:23'!
backBreadthFirstDoVertex: aVertexProcessingBlock doEdge: anEdgeProcessingBlock startingVertices: aSet1 visitedVertices: aSet2
"Based on BFS(Breath First Search) algorithm"
	| aboutToVisit visited|
	visited := aSet2 copy.
	aboutToVisit := aSet1 copy.

	[ aboutToVisit do: [:vertex| aVertexProcessingBlock value: vertex].
	   visited addAll: aboutToVisit.
	   aboutToVisit do: [:vertex| vertex edgesThatCanEnter do: [:leavingEdge| anEdgeProcessingBlock value: leavingEdge].
						aboutToVisit addAll: vertex backNeighbors ].
	   aboutToVisit := 
		aboutToVisit select:
			[:vertex| (self vertices includes: vertex) and:
				[(visited includes: vertex) not]].
	aboutToVisit isEmpty ] whileFalse! !

!Graph methodsFor: 'visiting operation' stamp: 'hjo 9/18/2009 16:23'!
backNeighboringVertices
"a set of vertices that is connected to the vertex of this graph but not included in this graph"

	^(self vertices dump: [:vertex| vertex backNeighbors ])
		reject: [:vertex| self vertices includes: vertex]! !

!Graph methodsFor: 'connecting' stamp: 'hjo 9/18/2009 16:24'!
connectVertices: collection1 andVertices: collection2

	collection1 do: [:eachNode | eachNode connectAll: collection2 ].! !

!Graph methodsFor: 'connecting' stamp: 'hjo 9/18/2009 16:24'!
connectVertices: collection1 toVertices: collection2

	collection1 do: [:eachNode | eachNode connectToAll: collection2 ].! !

!Graph methodsFor: 'querying' stamp: 'hjo 9/20/2009 15:39'!
connectedComponents

	| components unvisitedVertices |
	unvisitedVertices := self vertices copy.
	components := Set new.
	[	| visitor |
		(visitor := BreadthFirstGraphVisitor new)
			currentVertices: (Array with: unvisitedVertices anyOne);
			run.
		components add: (self class vertices: visitor visitedVertices).
		unvisitedVertices removeAll: visitor visitedVertices.

		unvisitedVertices isEmpty
	] whileFalse.
	^components! !

!Graph methodsFor: 'vertex' stamp: 'hjo 9/22/2009 14:29'!
createVertexAndAddIt

	| newVertex |
	newVertex := self class defaultVertexClass new.
	self addVertex: newVertex.
	^newVertex! !

!Graph methodsFor: 'visiting operation' stamp: 'hjo 9/18/2009 16:25'!
depthFirstDo: aVertexProcessingBlock startingAt: aVertex

	"Based on Nonrecursive DFS(Depth First Search) algorithm"
	| visited currentVertex vertexStack aboutToVisit |
	currentVertex := aVertex.
	visited := Set new: self vertices size.
	vertexStack := Stack new.
	vertexStack push: currentVertex.

	[ aboutToVisit := currentVertex neighbors reject:
		[:vertex| (self vertices excludes: vertex) or:
			[(visited includes: vertex) or: [vertexStack includes: vertex]] ].
	aboutToVisit isEmpty
		ifTrue: [ aVertexProcessingBlock value: currentVertex.
			visited add: currentVertex.
			currentVertex := vertexStack pop]
		ifFalse: [ vertexStack pushAll: aboutToVisit ].
	 vertexStack isEmpty ] whileFalse! !

!Graph methodsFor: 'querying' stamp: 'hjo 9/18/2009 16:25'!
directionedEdges

	^self edges select: [:each| each isDirectedEdge ]! !

!Graph methodsFor: 'querying' stamp: 'hjo 9/18/2009 16:26'!
directionlessEdge

	^self edges reject: [:each| each isDirectedEdge ]! !

!Graph methodsFor: 'drawing' stamp: 'hjo 9/18/2009 16:26'!
drawOn: aCanvas

	self edges do: [:edge| edge drawOn: aCanvas].
		self vertices do: [:vertex|	vertex drawOn: aCanvas]
! !

!Graph methodsFor: 'drawing' stamp: 'hjo 9/18/2009 16:27'!
drawOn: aCanvas withMapper: aMapper

	self edges do: [:edge| edge drawOn: aCanvas withMapper: aMapper].
		self vertices do: [:positionedVertex|
			positionedVertex drawOn: aCanvas withMapper: aMapper]
! !

!Graph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:27'!
edges

	| edges |
	edges := Set new.
	self vertices do: [:v| edges addAll: v edges].
	^edges! !

!Graph methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 16:27'!
finalizeAllVertices

	self vertices do: [:vertex| vertex finalize ]! !

!Graph methodsFor: 'data structures' stamp: 'hjo 9/18/2009 16:28'!
incidenceMatrix

	self notImplemented! !

!Graph methodsFor: 'initialize-release' stamp: 'hjo 9/26/2009 20:36'!
initialize

	self removeAllVertices! !

!Graph methodsFor: 'testing' stamp: 'hjo 11/30/2009 12:48'!
isComplete

"This is not enough for testing completeness of a graph."
	| verticesSize |
	verticesSize := self vertices size.
	self revisit: 'self connectedComponents size is boolean expression?'.
	^self edges size >= verticesSize * (verticesSize - 1) / 2 and: [ self connectedComponents size ].
	! !

!Graph methodsFor: 'testing' stamp: 'hjo 9/20/2009 15:39'!
isConnected
"A graph is connected if there is a path from every vertex to every other vertex in the graph"
	| visitor |
	visitor := BreadthFirstGraphVisitor new.
	visitor currentVertices: (Array with: self vertices anyOne).
	visitor run.
	^visitor visitedVertices equalElementsTo: self vertices
	"| connectedVertices |
	connectedVertices := Set new.
	self
		breadthFirstDo: [:vertex| connectedVertices add: vertex ]
		startingAt: self vertices any.
	^self vertices equals: connectedVertices"! !

!Graph methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:30'!
isDense

	^self isSparse not! !

!Graph methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:30'!
isDigraph
"DiGraph is a Graph whose edges are all directed edge.
Two edges connecting the same two points i,j are now permitted, provided they have opposite directions, they are (i,j) and (j,i)"
	^self edges allSatisfy: [:e| e isDirectedEdge ]! !

!Graph methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:30'!
isSparse

	| verticesSize |
	verticesSize := self vertices size.
	^self edges size < (verticesSize * verticesSize log)! !

!Graph methodsFor: 'testing' stamp: 'hjo 9/18/2009 16:30'!
isSubgraphOf: aGraph

"A subgraph or subdigraph of a given graph or digraph G = (V, E), resepectively, is a graph or digraph obtained by deleting some of the edges and vertices of G, retaining the other edges of G(together with their pairs of endpoints)."
	^(self vertices isSubcollectionOf: aGraph vertices)
		and: [ self edges isSubcollectionOf: aGraph edges]! !

!Graph methodsFor: 'testing' stamp: 'hjo 9/20/2009 15:40'!
isTree
"A graph with no cycles is called a tree"

	| visitedEdges |
	visitedEdges := Set new.
	BreadthFirstGraphVisitor new
		currentVertices: (Array with: self vertices anyOne);
		edgeOperation: [:edge| visitedEdges add: edge ];
		run.
	^visitedEdges equalElementsTo: self edges
	
	"|visited|
	visited := Set new.
	self
		breadthFirstDo:
			[:vertex|(self vertices includes: vertex)
				ifTrue: [^false]
				ifFalse: [visited add: vertex]]
		startingAt: self vertices asArray first.
	^true"! !

!Graph methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 16:30'!
letGoVertex: aVertex

	aVertex finalize! !

!Graph methodsFor: 'querying' stamp: 'hjo 9/18/2009 16:31'!
neighboringVertices
"a set of vertices that is connected to the vertex of this graph but not included in this graph"

	^(self vertices dump: [:vertex| vertex neighbors ])
		reject: [:vertex| self vertices includes: vertex]! !

!Graph methodsFor: 'vertex' stamp: 'hjo 9/18/2009 16:31'!
removeAllVertices

	vertices := Set new.! !

!Graph methodsFor: 'vertex' stamp: 'hjo 9/18/2009 16:31'!
removeVertex: aVertex

	self vertices remove: aVertex ifAbsent: [self error: 'Can''t remove unexisting node'].! !

!Graph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:56'!
vertices

	^vertices! !

!Graph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 16:56'!
vertices: aCollection

	vertices := aCollection asSet! !

Graph subclass: #NamedGraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!NamedGraph classSide methodsFor: 'accessing' stamp: 'hjo 9/26/2009 19:51'!
defaultVertexClass

	^NamedVertex! !

!NamedGraph methodsFor: 'vertex' stamp: 'hjo 11/29/2009 16:14'!
addVertex: aVertex named: aString

	[^self vertices add: aVertex] ensure: [self name: aVertex as: aString]! !

!NamedGraph methodsFor: 'vertex' stamp: 'hjo 11/29/2009 16:56'!
addVertexNamed: aString

	^self
		addVertex: (self class defaultVertexClass new name: aString asString)
		named: aString! !

!NamedGraph methodsFor: 'vertex' stamp: 'hjo 11/29/2009 16:56'!
addVerticesNamed: aCollection

	^aCollection collect: [:vertexName| self addVertexNamed: vertexName ]! !

!NamedGraph methodsFor: 'connecting' stamp: 'hjo 11/29/2009 19:50'!
connectVertexNamed: vertexName1 andVertexNamed: vertexName2

	"Simplified version of connecting vertices by using its names."
	^(self vertexNamed: vertexName1) connect: (self vertexNamed: vertexName2).! !

!NamedGraph methodsFor: 'connecting' stamp: 'hjo 11/29/2009 19:50'!
connectVertexNamed: vertexName1 toVertexNamed: vertexName2

	^(self vertexNamed: vertexName1) connectTo: (self vertexNamed: vertexName2)! !

!NamedGraph methodsFor: 'querying' stamp: 'hjo 9/26/2009 20:36'!
getOrAndNewVertexNamed: aByteString 

	| newVertex |
	^vertices detect: [:each| each name = aByteString ] 
		ifNone: 
			[newVertex := self class defaultVertexClass new.
			self addVertex: newVertex named: aByteString.
			newVertex]! !

!NamedGraph methodsFor: 'testing' stamp: 'hjo 9/26/2009 20:26'!
hasVertexNamed: aString

	^vertices anySatisfy: [:v| v name = aString ]! !

!NamedGraph methodsFor: 'names' stamp: 'hjo 9/26/2009 19:57'!
name: aNamedVertex as: aString

	aNamedVertex name: aString.
	^aNamedVertex! !

!NamedGraph methodsFor: 'names' stamp: 'hjo 9/26/2009 19:58'!
nameOf: aNamedVertex

	^aNamedVertex name! !

!NamedGraph methodsFor: 'names' stamp: 'hjo 11/29/2009 15:59'!
nodeNames
	
	^self vertexNames! !

!NamedGraph methodsFor: 'querying' stamp: 'hjo 9/26/2009 19:58'!
vertexNamed: aString

	^self vertices detect: [:vertex| vertex name = aString ]! !

!NamedGraph methodsFor: 'names' stamp: 'hjo 11/29/2009 15:59'!
vertexNames
	
	^vertices collect: [:each| each name ]! !

!NamedGraph methodsFor: 'finding' stamp: 'hjo 11/29/2009 19:43'!
verticesNamed: aCollection

	^aCollection collect: [:vertexName| self vertexNamed: vertexName]! !

Graph subclass: #PlotGraph
	instanceVariableNames: 'edgeColor edgeStyle edgeThickness vertexColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Graphics'!

!PlotGraph classSide methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:38'!
defaultVertexClass

	^PlotVertex! !

!PlotGraph classSide methodsFor: 'examples' stamp: 'hjo 9/18/2009 19:38'!
example

	|g|
	g := super example.
	g vertices do: [:vv| vv position: 20@30].
	^g! !

!PlotGraph methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 19:36'!
createOneThreadOfEdgesConnectingAllVertices: aSequenceableCollection

	|previousVertex vertexStream|
	vertexStream := aSequenceableCollection readStream.
	previousVertex := vertexStream next.
	[vertexStream atEnd] whileFalse:[ |nextVertex|
		nextVertex := vertexStream next.
		previousVertex connect: nextVertex.
		previousVertex := nextVertex].
	self addAllVertices: aSequenceableCollection! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:36'!
edgeColor

	^edgeColor! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:37'!
edgeColor: aColor

	edgeColor := aColor! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:36'!
edgeStyle

	^edgeStyle! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:36'!
edgeStyle: aSymbol

	edgeStyle := aSymbol! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:36'!
edgeThickness

	^edgeThickness! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:36'!
edgeThickness: anInteger

	edgeThickness := anInteger! !

!PlotGraph methodsFor: 'operation' stamp: 'hjo 9/18/2009 19:37'!
prepareDrawing

	self edges do: [:edge| edge color: edgeColor; thickness: edgeThickness; type: edgeStyle].
	self vertices do: [:vertex|vertex color:  vertexColor]! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:38'!
vertexColor

	^vertexColor! !

!PlotGraph methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:38'!
vertexColor: aColor

	vertexColor := aColor
! !

Object subclass: #GraphPathAbstract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

GraphPathAbstract subclass: #GraphPath
	instanceVariableNames: 'pathFrom lastVertex edgeToLastVertex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!GraphPath commentStamp: 'hjo 9/18/2009 15:10' prior: 0!
Original name on Dolphin 4 was 'Path'.  It collided so I renamed.!

!GraphPath classSide methodsFor: 'examples' stamp: 'hjo 12/17/2009 12:53'!
example

	| e1 e2 e3 v1 v2 v3 v4 p | 
	e1 := Edge new.
	e2 := Edge new.
	e3 := Edge new.

	v1 := Vertex new.
	v2 := Vertex new.
	v3 := Vertex new.
	v4 := Vertex new.

	v1 name: 'v1'.
	v2 name: 'v2'.
	v3 name: 'v3'.
	v4 name: 'v4'.

	e1 connect: v1 and: v2.
	e2 connect: v2 and: v3.
	e3 connect: v3 and: v4.

	p := self new.
	p startVertex: v1.
	p endVertex: v4.
	p edgeSequence: (OrderedCollection with: e1 with: e2 with: e3).
	^p! !

!GraphPath classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 17:16'!
from: aVertex to: bVertex through: anEdgeSequence

	^self new
		startVertex: aVertex;
		endVertex: bVertex;
		edgeSequence: anEdgeSequence;
		yourself! !

!GraphPath methodsFor: 'testing' stamp: 'hjo 11/7/2009 11:29'!
containsSubPathFrom: aCostVertex to: aCostVertex2 

	^(self includesVertex: aCostVertex2) and: [ self includesVertex: aCostVertex ]! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 16:59'!
cost

	^pathFrom cost + edgeToLastVertex cost! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 14:02'!
edgeToLastVertex

	^ edgeToLastVertex! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 14:02'!
edgeToLastVertex: anEdge

	edgeToLastVertex := anEdge! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 18:19'!
edges

	^pathFrom edges , (Array with: edgeToLastVertex)! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/21/2009 15:36'!
firstVertex

	^self pathFrom firstVertex! !

!GraphPath methodsFor: 'testing' stamp: 'hjo 11/7/2009 11:23'!
includesVertex: aCostVertex 

	^lastVertex == aCostVertex or: [pathFrom includesVertex:  aCostVertex]! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 14:03'!
lastVertex

	^ lastVertex! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 14:03'!
lastVertex: aVertex

	lastVertex := aVertex! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 14:03'!
pathFrom

	^ pathFrom! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 14:03'!
pathFrom: aGraphPath

	pathFrom := aGraphPath! !

!GraphPath methodsFor: 'printing' stamp: 'hjo 11/9/2009 14:44'!
printOn: aStream

	pathFrom printOn: aStream.
	aStream nextPutAll: '--'.
	aStream print: edgeToLastVertex cost.
	aStream nextPutAll: '--'.
	aStream nextPut: $(.
	aStream nextPutAll: lastVertex graphPathSymbolText.
	aStream nextPut: $)! !

!GraphPath methodsFor: 'creating path' stamp: 'hjo 11/7/2009 17:36'!
subPathFrom: aCostVertex to: aCostVertex2
	
	^aCostVertex2 == lastVertex
		ifTrue: [ (pathFrom subPathToEndFrom: aCostVertex) extendedTo: lastVertex via: edgeToLastVertex ]
		ifFalse: [ "What to do here?" self halt "Why is this break point not reached?" ]	
! !

!GraphPath methodsFor: 'creating path' stamp: 'hjo 11/7/2009 15:18'!
subPathToEndFrom: aCostVertex 

	^aCostVertex == lastVertex
		ifTrue: [ GraphPathWithOneVertex vertex: lastVertex ]
		ifFalse: [ (pathFrom subPathToEndFrom: aCostVertex) extendedTo: lastVertex via: edgeToLastVertex]! !

!GraphPath methodsFor: 'accessing' stamp: 'hjo 9/22/2009 17:08'!
vertices

	^pathFrom vertices , (Array with: lastVertex)! !

!GraphPathAbstract methodsFor: 'as yet unclassified' stamp: 'hjo 11/9/2009 02:00'!
costFrom: aCostVertex to: aCostVertex2 

	^(self subPathFrom: aCostVertex to: aCostVertex2) cost! !

!GraphPathAbstract methodsFor: 'creating path' stamp: 'hjo 11/12/2009 16:30'!
extendedTo: aVertex via: anEdge 

	| newPath |
	self assert: [ anEdge canFlowFrom: self lastVertex to: aVertex ] description: 'Given edge cannot connect from me to aVertex'.
	newPath := GraphPath new.
	newPath pathFrom: self.
	newPath lastVertex: aVertex.
	newPath edgeToLastVertex: anEdge.
	^newPath! !

GraphPathAbstract subclass: #GraphPathWithOneVertex
	instanceVariableNames: 'vertex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!GraphPathWithOneVertex classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/22/2009 15:23'!
vertex: aVertex 

	^self new vertex: aVertex! !

!GraphPathWithOneVertex methodsFor: 'testing' stamp: 'hjo 11/9/2009 01:53'!
containsSubPathFrom: aCostVertex to: aCostVertex2 
	
	^aCostVertex == aCostVertex2 and: [ vertex == aCostVertex ]! !

!GraphPathWithOneVertex methodsFor: 'accessing' stamp: 'hjo 9/22/2009 16:59'!
cost
	
	^0! !

!GraphPathWithOneVertex methodsFor: 'accessing' stamp: 'hjo 9/22/2009 18:20'!
edges

	^Array new! !

!GraphPathWithOneVertex methodsFor: 'accessing' stamp: 'hjo 9/21/2009 15:24'!
firstVertex

	^self vertex! !

!GraphPathWithOneVertex methodsFor: 'testing' stamp: 'hjo 11/7/2009 11:52'!
includesVertex: aCostVertex 

	self revisit: 'Rename this method. includes: uses equality not identity for comparison.'.
	^vertex == aCostVertex! !

!GraphPathWithOneVertex methodsFor: 'accessing' stamp: 'hjo 9/21/2009 15:24'!
lastVertex

	^self vertex! !

!GraphPathWithOneVertex methodsFor: 'printing' stamp: 'hjo 11/9/2009 16:24'!
printOn: aStream

	aStream nextPut: $(.
	aStream nextPutAll: vertex graphPathSymbolText.
	aStream nextPut: $)! !

!GraphPathWithOneVertex methodsFor: 'creating path' stamp: 'hjo 11/7/2009 11:55'!
subPathFrom: aCostVertex to: aCostVertex2 

	vertex == aCostVertex
		ifTrue: [ ^self class vertex: aCostVertex ].
	vertex == aCostVertex2
		ifTrue: [ ^self class vertex: aCostVertex2 ].
	self error: 'What should I return?'! !

!GraphPathWithOneVertex methodsFor: 'creating path' stamp: 'hjo 11/9/2009 12:38'!
subPathToEndFrom: aCostVertex 

	^self copy! !

!GraphPathWithOneVertex methodsFor: 'accessing' stamp: 'hjo 9/21/2009 15:23'!
vertex
	"Answer the value of vertex"

	^ vertex! !

!GraphPathWithOneVertex methodsFor: 'accessing' stamp: 'hjo 9/21/2009 15:23'!
vertex: anObject
	"Set the value of vertex"

	vertex := anObject! !

!GraphPathWithOneVertex methodsFor: 'accessing' stamp: 'hjo 9/22/2009 17:09'!
vertices

	^Array with: vertex! !

Object subclass: #StateMachine
	instanceVariableNames: 'currentState nameTable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-StateMachine'!

!StateMachine classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:40'!
defaultVertexClass

	^State! !

!StateMachine classSide methodsFor: 'examples' stamp: 'hjo 9/18/2009 19:40'!
example

	"Example from the book 'Evolutionary Computation' page 61 by David B. Fogel"
	| stateMachine stateA stateB stateC |

	"Create StateMachine and States"
	stateMachine := self new.
	stateMachine addVertex: (stateA := self defaultVertexClass new name: 'A').
	stateMachine addVertex: (stateB := self defaultVertexClass new name: 'B').
	stateMachine addVertex: (stateC := self defaultVertexClass new name: 'C').
	
	"Specify relationships between states"
	stateA
		for: 0 give: #beta be: stateB;
		for: 1 give: #beta be: stateA.
	stateB
		for: 0 give: #gamma be: stateB;
		for: 1 give: #alpha be: stateC.
	stateC
		for: 0 give: #beta be: stateB;
		for: 1 give: #gamma be: stateA.
	
	stateMachine state: stateA.
	^stateMachine! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:39'!
input: anObject

	| output |
	output := self state outputFor: anObject.
	self state: (self state nextStateFor: anObject).
	^output! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:40'!
inputs: aSeqCollection

	| inStream outStream |

	inStream := aSeqCollection readStream.
	outStream := Array writeStream.
	
	[ inStream atEnd ]
		whileFalse:
		[ outStream nextPut: (self input: (inStream next)) ].
	^outStream contents! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:40'!
state

	^currentState! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:40'!
state: aState

	currentState := aState! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:40'!
stateName

	^nameTable keyAtValue: self state! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:40'!
stateNamed: aString

	self state: (self vertexNamed: aString)! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:40'!
states

	^self vertices! !

Object subclass: #StochasticStateMachine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Stochastic'!

!StochasticStateMachine classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:47'!
defaultVertexClass

	^ProbableState! !

!StochasticStateMachine classSide methodsFor: 'examples' stamp: 'hjo 11/29/2009 16:17'!
example

	|ssm|
	self revisit: 'This method may raise error, since addVertex:named: being moved from Graph to NamedGraph.'.
	ssm := self new.
	ssm
		addVertex: (self defaultVertexClass new name: 'hungry')  named: 'hungry';
		addVertex: (self defaultVertexClass new name: 'hunt') named: 'hunt';
		addVertex: (self defaultVertexClass new name: 'normal') named: 'normal';
		addVertex: (self defaultVertexClass new name: 'starved') named: 'starved';
		addVertex: (self defaultVertexClass new name: 'tired') named: 'tired'.

	(ssm vertexNamed: 'hungry')
		for: 0.6 give: 'I need food.' be: (ssm vertexNamed: 'hungry');
		for: 0.1 give: 'I''m starving.' be: (ssm vertexNamed: 'starved');
		for: 0.2 give: 'I found a prey!!!!' be: (ssm vertexNamed: 'hunt');
		for: 0.1 give: 'I have no energy do anything.' be: (ssm vertexNamed: 'tired').

	(ssm vertexNamed: 'starved')
		for: 1.0 give: 'rest in peace' be: (ssm vertexNamed: 'starved').

	(ssm vertexNamed: 'hunt')
		for: 0.5 give: 'I''m chasing the prey.' be: (ssm vertexNamed: 'hunt');
		for: 0.2 give: 'I lost it' be: (ssm vertexNamed: 'tired');
		for: 0.3 give: 'I got it!!!! Good Taste!!!!' be: (ssm vertexNamed: 'normal').

	(ssm vertexNamed: 'tired')
		for: 0.4 give: 'I''m so tired.' be: (ssm vertexNamed: 'tired');
		for: 0.3 give: 'I need food to stand up.' be: (ssm vertexNamed: 'hungry');
		for: 0.3 give: 'Not gonna miss this time.' be: (ssm vertexNamed: 'hunt').
		
	(ssm vertexNamed: 'normal')
		for: 0.8 give: 'I''m a happy one.' be: (ssm vertexNamed: 'normal');
		for: 0.1 give: 'I hunt for fun' be: (ssm vertexNamed: 'hunt');
		for: 0.1 give: 'I feel hungry' be: (ssm vertexNamed: 'hungry').

	ssm state: (ssm vertexNamed: 'normal').
	^ssm! !

!StochasticStateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:47'!
globalRanom

	^Collection randomForPicking next! !

!StochasticStateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:47'!
nextState

	^self input: self globalRandom! !

!StochasticStateMachine methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:47'!
nextState: anInteger

	^self inputs: (self globalRandom next: anInteger)
! !

Object subclass: #Vertex
	instanceVariableNames: 'enteringEdges leavingEdges twoWayEdges masterObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Core'!

!Vertex commentStamp: '<historical>' prior: 0!
Vertex is a material to form a Graph. Vertex can be connect by Edge and it knows all Edges that connects to it.!

Vertex subclass: #CostVertex
	instanceVariableNames: 'minimumCostPaths'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!CostVertex classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/21/2009 14:47'!
designatedEdgeClass

	^CostEdge! !

!CostVertex methodsFor: 'shortest path finding' stamp: 'hjo 11/7/2009 19:24'!
clearMinimumCostPaths

	self revisit: 'This method should be private. You do not have to call this method, if you use minimumCostPathsFrom: series instead.'.
	minimumCostPaths := MinClub new.
	minimumCostPaths evaluator: [:path| path cost ]! !

!CostVertex methodsFor: 'shortest path finding' stamp: 'hjo 11/18/2009 20:13'!
computeMinimumCostGraphPathsFrom: aCostVertex 

	Dijkstra computeMinimumCostGraphPathsFrom: aCostVertex to: self! !

!CostVertex methodsFor: 'connecting' stamp: 'hjo 9/28/2009 12:53'!
connect: aVertex cost: aNumber

	^(self connect: aVertex)
		cost: aNumber;
		yourself! !

!CostVertex methodsFor: 'connecting' stamp: 'hjo 9/25/2009 19:14'!
connectTo: aVertex cost: aNumber

	^(self connectTo: aVertex)
		cost: aNumber;
		yourself! !

!CostVertex methodsFor: 'initialize-release' stamp: 'hjo 9/26/2009 17:46'!
initialize

	super initialize.
	self clearMinimumCostPaths! !

!CostVertex methodsFor: 'initialize-release' stamp: 'hjo 11/11/2009 08:27'!
initializeMinimumCostPathsClubWithSource: aCostVertex 

	minimumCostPaths := MinClub new.
	minimumCostPaths evaluator: [:path| path costFrom: aCostVertex to: self ]! !

!CostVertex methodsFor: 'shortest path finding - recycling' stamp: 'hjo 11/7/2009 18:55'!
minCostPathClubFrom: aCostVertex 

	| metaPaths subMinClub |
	self recyclinglyComputeMinimumCostGraphPathsFrom: aCostVertex.
	metaPaths := minimumCostPaths members select: [:each| each containsSubPathFrom: aCostVertex to: self ].
	subMinClub := MinClub new.
	subMinClub evaluator: [:path| path costFrom: aCostVertex to: self ].
	subMinClub putAll: (	metaPaths collect: [:each| each subPathFrom: aCostVertex to: self ]
).	
	^subMinClub! !

!CostVertex methodsFor: 'accessing' stamp: 'hjo 9/25/2009 18:44'!
minimumCost

	^minimumCostPaths minimum! !

!CostVertex methodsFor: 'shortest path finding - recycling' stamp: 'hjo 11/7/2009 18:55'!
minimumCostFrom: aCostVertex 

	^(self minCostPathClubFrom: aCostVertex ) minimum! !

!CostVertex methodsFor: 'accessing' stamp: 'hjo 9/23/2009 01:14'!
minimumCostPaths

	^minimumCostPaths! !

!CostVertex methodsFor: 'accessing' stamp: 'hjo 11/11/2009 08:28'!
minimumCostPathsClub

	^minimumCostPaths! !

!CostVertex methodsFor: 'shortest path finding - recycling' stamp: 'hjo 11/7/2009 18:56'!
minimumCostPathsFrom: aCostVertex 

	^(self minCostPathClubFrom: aCostVertex) members! !

!CostVertex methodsFor: 'shortest path finding - recycling' stamp: 'hjo 11/7/2009 18:45'!
recyclinglyComputeMinimumCostGraphPathsFrom: aCostVertex 

	(self minimumCostPaths members anySatisfy: [:each| each firstVertex == aCostVertex])
		ifTrue: [ "Can recycle." ^self ].
	(self minimumCostPaths members anySatisfy: [:each| each includesVertex: aCostVertex])
		ifTrue: [ "Can recycle." ^self ].
	"Hopeless to recycle the precalculated results."
	self breadthFirstDoVertex: [:vertex| vertex clearMinimumCostPaths].
	self computeMinimumCostGraphPathsFrom: aCostVertex ! !

CostVertex subclass: #NamedCostVertex
	instanceVariableNames: 'vertexName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!NamedCostVertex methodsFor: 'printing' stamp: 'hjo 11/7/2009 12:55'!
detailPrintString

	^vertexName isNil
		ifTrue: [ 'nameless' ]
		ifFalse: [ vertexName  ]! !

!NamedCostVertex methodsFor: 'printing' stamp: 'hjo 12/2/2009 19:54'!
printOn: aStream
	
	aStream nextPutParenthesisAround: [ 
		aStream nextPutAll: self detailPrintString
		]! !

!NamedCostVertex methodsFor: 'accessing' stamp: 'hjo 11/7/2009 12:53'!
vertexName

	^vertexName! !

!NamedCostVertex methodsFor: 'accessing' stamp: 'hjo 11/7/2009 12:53'!
vertexName: aString

	vertexName := aString! !

Vertex subclass: #DrawableVertex
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Graphics'!

!DrawableVertex methodsFor: 'printing' stamp: 'hjo 9/19/2009 17:40'!
detailPrintString

	|string|
	string := super detailPrintString.
	position notNil ifTrue: [ string size = 0 ifFalse: [string := string, ' '].
						string := string, position printString].
	^string! !

!DrawableVertex methodsFor: 'drawing' stamp: 'hjo 9/19/2009 17:38'!
drawOn: aCanvas

	|icon|
	icon := Presenter icon.
	icon drawOn: aCanvas at: position asPoint-((icon extent/2) floor)! !

!DrawableVertex methodsFor: 'accessing' stamp: 'hjo 9/19/2009 17:38'!
position

	^position! !

!DrawableVertex methodsFor: 'accessing' stamp: 'hjo 9/19/2009 17:38'!
position: aVector

	position := aVector! !

!DrawableVertex methodsFor: 'drawing' stamp: 'hjo 9/19/2009 17:38'!
rectangleToInvalidate

	|theRectangle|
	theRectangle := self icon rectangle center: position.
	self edges do: [:edge|theRectangle := theRectangle union: edge rectangleToInvalidate].
	^theRectangle! !

Vertex subclass: #NamedVertex
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Extended'!

!NamedVertex methodsFor: 'accessing' stamp: 'hjo 9/26/2009 19:54'!
name

	^name! !

!NamedVertex methodsFor: 'accessing' stamp: 'hjo 9/26/2009 19:54'!
name: aString

	name := aString! !

Vertex subclass: #PlotVertex
	instanceVariableNames: 'shape color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Graphics'!

!PlotVertex classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:50'!
designatedEdgeClass

	^PlotEdge! !

!PlotVertex methodsFor: 'state changing' stamp: 'hjo 9/18/2009 20:35'!
beCircle

	shape := #circle! !

!PlotVertex methodsFor: 'state changing' stamp: 'hjo 9/18/2009 20:35'!
beNoShape

	shape = #none
! !

!PlotVertex methodsFor: 'state changing' stamp: 'hjo 9/18/2009 20:35'!
beSquare

	shape := #square! !

!PlotVertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 20:35'!
color

	^color! !

!PlotVertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 20:35'!
color: aColor

	color := aColor! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:36'!
drawCircleOn: aCanvas

"	Dolphin 4 code
	|aBrush|
	aBrush := aCanvas brush: (Brush color: color).
	aCanvas ellipse: self iconRectangle.
	aCanvas brush: aBrush."! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:36'!
drawCircleOn: aCanvas withMapper: aMapper

	"Dolphin 4 code
	|aBrush|
	aBrush := aCanvas brush: (Brush color: color).
	aCanvas ellipse: (self iconRectangle magnifiedBy: aMapper ratio).
	aCanvas brush: aBrush."! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:36'!
drawOn: aCanvas

	"Dolphin 4 code
	|topLeft|
	topLeft := position asPoint-((self iconSize/2) floor).
	shape = #none ifTrue: [^self].
	shape = #square ifTrue: [^self drawSquareOn: aCanvas].
	shape = #circle ifTrue: [^self drawCircleOn: aCanvas]."! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:36'!
drawOn: aCanvas withMapper: aMapper

"	Dolphin 4 code
	|topLeft|
	topLeft := position asPoint-((self iconSize/2) floor).
	shape = #none ifTrue: [^self].
	shape = #square ifTrue: [^self drawSquareOn: aCanvas withMapper: aMapper].
	shape = #circle ifTrue: [^self drawCircleOn: aCanvas withMapper: aMapper]."! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:37'!
drawSquareOn: aCanvas

"	Dolphin4 code
	|aBrush|
	aBrush := aCanvas brush: (Brush color: color).
	aCanvas rectangle: self iconRectangle.
	aCanvas brush: aBrush."! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:37'!
drawSquareOn: aCanvas withMapper: aMapper

"	Dolphin 4 code
	|aBrush|
	aBrush := aCanvas brush: (Brush color: color).
	aCanvas rectangle: (self iconRectangle magnifiedBy: aMapper ratio).
	aCanvas brush: aBrush."! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:37'!
iconRectangle

	^(0@0 corner: 10@10) center: self position.! !

!PlotVertex methodsFor: 'drawing' stamp: 'hjo 9/18/2009 20:37'!
iconSize

	^10@10! !

!PlotVertex methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 20:37'!
initialize

	super initialize.
	shape := #circle.
	color := Color blue.! !

!PlotVertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 20:37'!
shape

	^shape! !

!PlotVertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 20:37'!
shape: aSymbol
	
	shape := aSymbol! !

Vertex subclass: #StateAbstract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-StateMachine'!

StateAbstract subclass: #ProbableState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-Stochastic'!

!ProbableState classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:56'!
designatedEdgeClass

	^StochasticEdge! !

!ProbableState methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:55'!
for: transProbability give: output be: aProbableState

	(self connectTo: aProbableState)
		probability: transProbability;
		output: output! !

!ProbableState methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:55'!
normalizeConditionaryProbabilities

	|sum|
	sum := self sumOfConditionaryProbabilities.
	self leavingEdges do: [:edge | edge probability: (edge probability / sum)]! !

!ProbableState methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:55'!
stateEdgeFor: aNumber

	| dice |
	dice := aNumber.
	^self leavingEdges
		detect:
		[:edge|
		dice<= edge probability
			ifTrue:
			[true]
			ifFalse:
			[dice := dice - edge probability. false]]! !

!ProbableState methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:55'!
sumOfConditionaryProbabilities

	^self leavingEdges inject: 0 into: [:sum :edge| edge probability + sum ]! !

StateAbstract subclass: #State
	instanceVariableNames: 'inputToEdgeTable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graph-StateMachine'!

!State classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 20:58'!
designatedEdgeClass

	^StateEdge! !

!State methodsFor: 'structuring' stamp: 'hjo 9/18/2009 20:56'!
for: input give: output be: aState

	(inputToEdgeTable
		at: input
		put: (self connectTo: aState)) output: output! !

!State methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 20:57'!
initialize

	super initialize.
	inputToEdgeTable := Dictionary new! !

!State methodsFor: 'querying' stamp: 'hjo 9/18/2009 20:57'!
stateEdgeFor: anObject

	^inputToEdgeTable at: anObject! !

!StateAbstract methodsFor: 'structuring' stamp: 'hjo 9/18/2009 20:50'!
for: input give: output be: aState

	self subclassResponsibility	! !

!StateAbstract methodsFor: 'querying' stamp: 'hjo 9/18/2009 20:51'!
nextStateFor: anObject

	^(self stateEdgeFor: anObject) terminalPoint! !

!StateAbstract methodsFor: 'querying' stamp: 'hjo 9/18/2009 20:51'!
outputFor: anObject

	^(self stateEdgeFor: anObject) output! !

!StateAbstract methodsFor: 'querying' stamp: 'hjo 9/18/2009 20:51'!
stateEdgeFor: anObject

	self subclassResponsibility! !

!Vertex classSide methodsFor: 'as yet unclassified' stamp: 'hjo 9/18/2009 19:20'!
designatedEdgeClass

	^Edge! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 17:16'!
addAllEdges: aCollection

	aCollection do: [:edge| self addEdge: edge ]
	"self edges addAll: aCollection"! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:02'!
addEdge: anEdge

	(anEdge hasDirection)
		ifTrue:
		[ self == anEdge terminalPoint 
			ifTrue:
			[ enteringEdges add: anEdge ]
			ifFalse: 
			[ leavingEdges add: anEdge ]
		"There was a Korean comment here."
		]
		ifFalse:
		[ twoWayEdges add: anEdge ].
	"self edges add: anEdge"! !

!Vertex methodsFor: 'querying' stamp: 'hjo 9/18/2009 18:02'!
backNeighbors

	^self edgesThatCanEnter collect: [:edge| edge otherEndPointOf: self ]
! !

!Vertex methodsFor: 'visiting operation' stamp: 'hjo 11/7/2009 18:28'!
breadthFirstDoVertex: vertexBlock

	| v |
	v := BreadthFirstGraphVisitor new.
	v currentVertices: (Array with: self).
	v vertexOperation: vertexBlock.
	v run.
	^v! !

!Vertex methodsFor: 'visiting operation' stamp: 'hjo 11/7/2009 18:28'!
breadthFirstDoVertex: vertexBlock doEdge: edgeBlock

	| v |
	v := BreadthFirstGraphVisitor new.
	v currentVertices: (Array with: self).
	v vertexOperation: vertexBlock.
	v edgeOperation: edgeBlock.
	v run.
	^v! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:02'!
connect: aVertex

	^self class designatedEdgeClass new connect: self and: aVertex! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:03'!
connectAll: aVertexCollection

	|newEdges|

	newEdges := OrderedCollection new.
	aVertexCollection do: [:vertex| newEdges add: (self connect: vertex) ].
	^newEdges! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:03'!
connectFrom: aVertex

	^self class designatedEdgeClass new connectFrom: aVertex to: self! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:03'!
connectFromAll: aVertexCollection

	|newEdges|
	newEdges := OrderedCollection new.
	aVertexCollection do: [:vertex| newEdges add: (self connectFrom: vertex) ].
	^newEdges! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:03'!
connectTo: aVertex

	^self class designatedEdgeClass new connectFrom: self to: aVertex! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:03'!
connectToAll: aVertexCollection

	|newEdges|
	newEdges := OrderedCollection new.
	aVertexCollection do: [:vertex| newEdges add: (self connectTo: vertex) ].
	^newEdges! !

!Vertex methodsFor: 'testing' stamp: 'hjo 9/18/2009 18:03'!
degree

	^self edges size! !

!Vertex methodsFor: 'printing' stamp: 'hjo 10/8/2009 16:15'!
detailPrintString

	^masterObject isNil
		ifTrue: [ String empty ]
		ifFalse: [ masterObject detailPrintStringForVertex ]! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 18:03'!
disconnectTo: aVertex

	(self edgesConnecting: aVertex) do: [:edge| edge disconnect]! !

!Vertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 18:04'!
edges

	^(twoWayEdges union: enteringEdges) union: leavingEdges! !

!Vertex methodsFor: 'querying' stamp: 'hjo 9/18/2009 18:04'!
edgesConnecting: aVertex

	^self edges select: [:edge| edge connects: self and: aVertex ]! !

!Vertex methodsFor: 'querying' stamp: 'hjo 9/18/2009 18:27'!
edgesThatCanEnter

	^twoWayEdges union: enteringEdges! !

!Vertex methodsFor: 'querying' stamp: 'hjo 9/18/2009 18:27'!
edgesThatCanLeave

	^twoWayEdges union: leavingEdges! !

!Vertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 18:27'!
enteringEdges 

	^enteringEdges! !

!Vertex methodsFor: 'finalization' stamp: 'hjo 9/18/2009 18:27'!
finalize

	self edges do: [:edge| edge letGo: self ]! !

!Vertex methodsFor: 'printing' stamp: 'hjo 11/9/2009 14:46'!
graphPathSymbolText

	^masterObject isNil
		ifTrue: [ '?' ]
		ifFalse: [ masterObject graphPathSymbolText ]! !

!Vertex methodsFor: 'testing' stamp: 'hjo 9/19/2009 19:00'!
hasEdge: anEdge 

	^self edges includes: anEdge! !

!Vertex methodsFor: 'initialize-release' stamp: 'hjo 9/18/2009 19:17'!
initialize

	super initialize.
	"self beFinalizable." "self toFinalizeSend: #finalize to: self with: nil."
	enteringEdges := Set new.
	leavingEdges := Set new.
	twoWayEdges := Set new! !

!Vertex methodsFor: 'testing' stamp: 'hjo 9/18/2009 19:18'!
isAdjacentWith: aVertex

	^self edges anySatisfy: [ :edge | edge connects: aVertex ]! !

!Vertex methodsFor: 'testing' stamp: 'hjo 9/18/2009 19:18'!
isConnectedTo: aVertex

	^self edges anySatisfy: 
		[:edge| edge connects: aVertex and: self ].! !

!Vertex methodsFor: 'testing' stamp: 'hjo 9/18/2009 19:18'!
isIsolatedVertex

	^self edges isEmpty! !

!Vertex methodsFor: 'testing' stamp: 'hjo 9/18/2009 19:18'!
isSelfConnected

	^self isConnectedTo: self! !

!Vertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:18'!
leavingEdges

	^leavingEdges! !

!Vertex methodsFor: 'accessing' stamp: 'hjo 9/19/2009 18:39'!
masterObject

	^masterObject! !

!Vertex methodsFor: 'accessing' stamp: 'hjo 9/19/2009 18:39'!
masterObject: anObject

	masterObject := anObject! !

!Vertex methodsFor: 'querying' stamp: 'hjo 9/18/2009 19:18'!
neighbors

	"OLD VERSION: ^self edges collect: [ :edge | edge otherEndPointOf: self ]"
	^self edgesThatCanLeave collect: [:edge| edge otherEndPointOf: self ]! !

!Vertex methodsFor: 'printing' stamp: 'hjo 10/8/2009 16:18'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutParenthesisAround: [ 
		masterObject isNil
			ifTrue: [aStream nextPutAll: 'nil']
			ifFalse: [masterObject printVertexDetailOn: aStream]
		]! !

!Vertex methodsFor: 'printing' stamp: 'hjo 11/9/2009 14:41'!
printVertexDescriptionForGraphPathOn: aWriteStream 

	masterObject printVertexDescriptionForGraphPathOn: aWriteStream! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 19:19'!
removeAllEdges

	self
		removeAllEnteringEdges;
		removeAllLeavingEdges;
		removeAllTwoWayEdges! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 19:19'!
removeAllEnteringEdges

	enteringEdges do: [:edge| edge disconnect]! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 19:19'!
removeAllLeavingEdges

	leavingEdges do: [:edge| edge disconnect]! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 19:19'!
removeAllTwoWayEdges

	twoWayEdges do: [:edge| edge disconnect]! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 19:19'!
removeEdge: anEdge

	"CAUTION: #removeEdge: does not nilify Edge referencing to this vertex"
	"self edges remove: anEdge ifAbsent: [ self error: 'Can''t remove unexisting edge' ]"
	(anEdge hasDirection)
		ifTrue:
		[ self == anEdge terminalPoint 
			ifTrue:
			[ enteringEdges remove: anEdge ]
			ifFalse:
			[ leavingEdges remove: anEdge ] ]
		ifFalse:
		[ twoWayEdges remove: anEdge ].! !

!Vertex methodsFor: 'connecting' stamp: 'hjo 9/18/2009 19:19'!
removeEdges: anEdgeCollection

	anEdgeCollection do: [:edge | self removeEdge: edge ]! !

!Vertex methodsFor: 'accessing' stamp: 'hjo 9/18/2009 19:19'!
twoWayEdges

	^twoWayEdges! !
